"""Type stubs for networkxr.generators."""

from networkxr.generators.classic import (
    barbell_graph as barbell_graph,
    complete_graph as complete_graph,
    cycle_graph as cycle_graph,
    empty_graph as empty_graph,
    path_graph as path_graph,
    star_graph as star_graph,
)
from networkxr.generators.fake import (
    fake_org_chart as fake_org_chart,
    fake_social_network as fake_social_network,
    fake_transaction_network as fake_transaction_network,
)

__all__: list[str]
