"""Type stubs for networkxr.generators.classic."""

from typing import Any

def empty_graph(n: int = 0, create_using: Any = None) -> Any: ...
def path_graph(n: int, create_using: Any = None) -> Any: ...
def cycle_graph(n: int, create_using: Any = None) -> Any: ...
def complete_graph(n: int, create_using: Any = None) -> Any: ...
def barbell_graph(m1: int, m2: int, create_using: Any = None) -> Any: ...
def star_graph(n: int, create_using: Any = None) -> Any: ...
