"""Type stubs for networkxr.exception."""

class NetworkXException(Exception): ...
class NetworkXError(NetworkXException): ...
class NetworkXPointlessConcept(NetworkXException): ...
class NetworkXAlgorithmError(NetworkXException): ...
class NetworkXUnfeasible(NetworkXAlgorithmError): ...
class NetworkXNoPath(NetworkXUnfeasible): ...
class NetworkXNoCycle(NetworkXUnfeasible): ...
class NetworkXUnbounded(NetworkXAlgorithmError): ...
class NodeNotFound(NetworkXException): ...
class HasACycle(NetworkXException): ...

class PowerIterationFailedConvergence(NetworkXAlgorithmError):
    num_iterations: int
    def __init__(self, num_iterations: int, *args: object) -> None: ...

class ExceededMaxIterations(NetworkXAlgorithmError): ...
class AmbiguousSolution(NetworkXException): ...
