"""Type stubs for networkxr.utils.misc."""

from typing import Any, Iterable, Iterator

def nodes_equal(nodes1: Any, nodes2: Any) -> bool: ...
def edges_equal(edges1: Any, edges2: Any, *, directed: bool = False) -> bool: ...
def graphs_equal(g1: Any, g2: Any) -> bool: ...
def pairwise(iterable: Iterable[Any]) -> Iterator[tuple[Any, Any]]: ...
def flatten(obj: Any, result: list[Any] | None = None) -> tuple[Any, ...]: ...
