"""Type stubs for networkxr.utils."""

from networkxr.utils.misc import (
    edges_equal as edges_equal,
    flatten as flatten,
    graphs_equal as graphs_equal,
    nodes_equal as nodes_equal,
    pairwise as pairwise,
)

__all__: list[str]
